/*************************************************************/
/*     FloatSpinBox (get org qt mailing list)                */
/*                                                           */
/*  Author : Joerg Ott(qt-mailing list),                     */
/*           T.Kojima                                        */
/*                                                           */
/*  Date   : 20071203                                        */
/*                                                           */
/*  Comment:                                                 */
/*                                                           */
/**************************************************************/
#include "FloatSpinBox.h"

#include <qspinbox.h>
#include <math.h>

/*!
  \class FloatSpinBox floatspinbox.h
  \brief reimplements QSpinBox for float values
*/

FloatSpinBox::FloatSpinBox( QWidget* parent, const char* name ,int decimals)
  : QSpinBox( int(-2e10), int(2e10), 10, parent, name ),dec(decimals)
{
  dVal = new QDoubleValidator( -2e8, 2e8, decimals, this, "dVal" );
  setValidator( dVal );	
  setValue( 0.0 );
}

FloatSpinBox::FloatSpinBox( float minValue, float maxValue, int decimals, 
			    float step, QWidget* parent, const char* name )
  : QSpinBox( int( minValue * pow( 10, decimals ) ), int( maxValue * pow( 10, decimals ) ), 
	      int( rint( step * pow( 10, decimals ) ) ), parent, name ),
    dec( decimals )
{
  dVal = new QDoubleValidator( minValue, maxValue, decimals, this, "dVal" );
  setValidator( dVal );	
  setValue( 0.0 );
}

FloatSpinBox::~FloatSpinBox()
{
}  

QString FloatSpinBox::mapValueToText ( int value )
{
  QString s;
  s.setNum( float( value )/ pow( 10, dec ), 'f', dec );
  return s;
}

int FloatSpinBox::mapTextToValue ( bool * ok )
{
  return int( cleanText().toFloat( ok ) * pow( 10, dec ) );
}

float FloatSpinBox::value() const
{
  return float( QRangeControl::value() ) / pow( 10, dec );
}

float FloatSpinBox::minValue() const
{
  return float( QRangeControl::minValue() ) / pow( 10, dec );
}

float FloatSpinBox::maxValue() const
{
  return float( QRangeControl::maxValue() ) / pow( 10, dec );
}

void FloatSpinBox::setValue( float value )
{
  QRangeControl::setValue( int( value *  pow( 10, dec ) ) );
}

void FloatSpinBox::setRange( float minValue, float maxValue )
{
	QRangeControl::setRange( int( minValue *  pow( 10, dec ) ), 
   int( maxValue *  pow( 10, dec ) ) );
  dVal->setRange( minValue, maxValue, 2 );

}

void FloatSpinBox::setAbsoluteMaxValue( float maxValue )
{
	maxValue *= pow( 10, dec ) ;
	setMaxValue((int)maxValue);
}

void FloatSpinBox::setAbsoluteMinValue( float minValue )
{
	minValue *= pow( 10, dec ) ;
	setMinValue((int)minValue);
}

void FloatSpinBox::valueChange()
{
  QSpinBox::valueChange();
  emit valueChanged( value() );
}

QSize FloatSpinBox::sizeHint() const
{
    QFontMetrics fm = fontMetrics();
    int h = fm.height();
    if ( h < 12 )       // ensure enough space for the button pixmaps
        h = 12;
    int w = 35;         // minimum width for the value
    int wx = fm.width( "  " );
    QString s;
    s.setNum( minValue(), 'f', dec );
    s.prepend( prefix() );
    s.append( suffix() );
    w = QMAX( w, fm.width( s ) + wx );
    s.setNum( maxValue(), 'f', dec );
    s.prepend( prefix() );
    s.append( suffix() );
    w = QMAX( w, fm.width( s ) + wx );
    s = specialValueText();
    w = QMAX( w, fm.width( s ) + wx );

    QSize r( h // buttons AND frame both sides
             + 6 // right/left margins
             + w, // widest value
             //+ frameWidth() * 2 // top/bottom frame
             + 4 // top/bottom margins
             + h // font height
             );
    return r;
}



